import click

from ..tasks import render, single_render
from ..version import __version__

CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])


@click.group(context_settings=CONTEXT_SETTINGS)
@click.version_option(__version__, "-v", "--version", message="version %(version)s")
def cli() -> None:
    """Reviews - A terminal UI Dashboard for monitoring code review requests.\n

    For feature requests or bug reports: https://github.com/apoclyps/reviews/issues
    """


@cli.command(help="Visualize code review requests as a Dashboard")
@click.option("-r", "--reload/--no-reload", default=True, is_flag=True)
def dashboard(reload: bool) -> None:
    """
    Command:\n
        reviews dashboard

    Usage:\n
        reviews dashboard --reload \n
        reviews dashboard --no-reload \n
    """

    click.echo("loading dashboard")

    if reload:
        render()
    else:
        single_render()


def main() -> None:
    """Entry point to CLI"""
    cli()



import os

from decouple import Csv, config

from reviews import config as application_config


def test_repository_configuration():
    os.environ[
        "REPOSITORY_CONFIGURATION"
    ] = "apoclyps/reviews, apoclyps/my-dev-space, apoclyps/magic-home"

    application_config.REPOSITORY_CONFIGURATION = config(
        "REPOSITORY_CONFIGURATION",
        cast=Csv(),
    )
    configuration = application_config.get_configuration()

    assert configuration == [
        ("apoclyps", "reviews"),
        ("apoclyps", "my-dev-space"),
        ("apoclyps", "magic-home"),
    ]



from pathlib import Path
from typing import Dict, List, Tuple

from decouple import Csv, config
from rich.color import ANSI_COLOR_NAMES

# Github Config
GITHUB_TOKEN = config("GITHUB_TOKEN", cast=str, default="")
GITHUB_USER = config("GITHUB_USER", cast=str, default="")
GITHUB_URL = config("GITHUB_URL", cast=str, default="https://api.github.com")
DEFAULT_PAGE_SIZE = config("DEFAULT_PAGE_SIZE", cast=int, default=100)

# Database Config
DATA_PATH = config("DATA_PATH", cast=str, default=f"{str(Path.home())}/.reviews")
FILENAME = config("FILENAME", cast=str, default="reviews.db")

# Application Config
DELAY_REFRESH = config("DELAY_REFRESH", cast=int, default=60)
ENABLE_PERSISTED_DATA = config("ENABLE_PERSISTED_DATA", cast=bool, default=False)
REPOSITORY_CONFIGURATION = config(
    "REPOSITORY_CONFIGURATION",
    cast=Csv(),
    default="apoclyps/reviews",
)
LABEL_CONFIGURATION = config(
    "LABEL_CONFIGURATION",
    cast=Csv(),
    default="blocked/orange,docker/blue,security/red,python/green",
)


def get_configuration() -> List[Tuple[str, str]]:
    """converts a comma seperated list of organizations/repositories into a list
    of tuples.
    """

    def _to_tuple(values: List[str]) -> Tuple[str, str]:
        return (values[0], values[1])

    return [
        _to_tuple(values=configuration.split(sep="/", maxsplit=1))
        for configuration in REPOSITORY_CONFIGURATION
    ]


def get_label_colour_map() -> Dict[str, str]:
    """converts a comma seperated list of organizations/repositories into a list
    of tuples.
    """

    def _preproc(label_colour: str) -> List[str]:
        return label_colour.lower().split(sep="/")

    return {
        label: f"[{colour}]"
        for label, colour in map(_preproc, LABEL_CONFIGURATION)
        if colour in ANSI_COLOR_NAMES
    }



from typing import Dict, List, Union

from github.PullRequest import PullRequest as ghPullRequest
from rich.table import Table

from . import config
from .layout import render_pull_request_table
from .source_control import GithubAPI, Label, PullRequest


class PullRequestController:
    """retrieve and store pull requests."""

    def __init__(self) -> None:
        self.client = GithubAPI()

    def retrieve_pull_requests(self, org: str, repository: str) -> Union[Table, None]:
        """Renders Terminal UI Dashboard"""

        title = f"{org}/{repository}"
        pull_requests = self.update_pull_requests(org=org, repository=repository)

        if not pull_requests:
            return None

        return render_pull_request_table(
            title=title,
            pull_requests=pull_requests,
            org=org,
            repository=repository,
        )

    def update_pull_requests(self, org: str, repository: str) -> List[PullRequest]:
        """Updates repository models."""

        def _get_reviews(pull_request: ghPullRequest) -> Dict[str, str]:
            """Inner function to retrieve reviews for a pull request"""
            reviews = pull_request.get_reviews()
            res, seen = {}, []

            for review in reviews:
                if review.user.login in seen or review.state == "COMMENTED":
                    continue
                res[review.user.login] = review.state
                seen.append(review.user.login)
            return res

        pull_requests = self.client.get_pull_requests(org=org, repo=repository)

        code_review_requests = []
        for pull_request in pull_requests:

            reviews = _get_reviews(pull_request=pull_request)

            if pull_request.user.login == config.GITHUB_USER:
                approved_by_me = "AUTHOR"
            else:
                approved_by_me = reviews.get(config.GITHUB_USER, "")  # NOQA: R1721

            approved_by_others = any(
                [
                    True
                    for user, status in reviews.items()
                    if user != config.GITHUB_USER and status == "APPROVED"
                ]
            )
            labels = [
                Label(name=label.name)
                for label in pull_request.get_labels()
                if label.name
            ]

            code_review_requests.append(
                PullRequest(
                    number=pull_request.number,
                    title=pull_request.title,
                    draft=pull_request.draft,
                    additions=pull_request.additions,
                    deletions=pull_request.deletions,
                    created_at=pull_request.created_at,
                    updated_at=pull_request.updated_at,
                    approved=approved_by_me,
                    approved_by_others=approved_by_others,
                    labels=labels,
                )
            )

        return code_review_requests
